@@ -14,7 +14,7 @@ write.fs.curv <- function(filepath, data) {
     MAGIC_FILE_TYPE_NUMBER = 16777215;
     num_verts = length(data);
     num_faces = length(data);   # Has no meaning.
-    values_per_vert = 1;
+    values_per_vert = 1L;
 
     if(guess.filename.is.gzipped(filepath, gz_extensions=c(".gz"))) {
         fh = gzfile(filepath, "wb");
@@ -26,7 +26,7 @@ write.fs.curv <- function(filepath, data) {
     writeBin(as.integer(num_verts), fh, endian = "big");
     writeBin(as.integer(num_faces), fh, endian = "big");
     writeBin(as.integer(values_per_vert), fh, endian = "big");
-    writeBin(data, fh, size = 4, endian = "big");
+    writeBin(data, fh, size = 4L, endian = "big");
     close(fh);
 }
 
@@ -51,14 +51,16 @@ fwrite3 <- function(filehandle, data) {
 }
 
 
-#' @title Write morphometry data in a format derived from the given file name (the file extension).
+#' @title Write morphometry data in a format derived from the given file name.
 #'
-#' @description Given data and a morphometry file name, derive the proper format and write the file.
+#' @description Given data and a morphometry file name, derive the proper format from the file extension and write the file.
 #'
 #' @param filepath, string. The full file name. The format to use will be derived from the last characters, the suffix. Supported suffixes are "mgh" for MGH format, "mgz" for MGZ format, everything else will be treated as curv format.
 #'
 #' @param data, numerical vector. The data to write.
 #'
+#' @param format character string, the format to use. One of c("auto", "mgh", "mgz", "curv"). The default setting "auto" will determine the format from the file extension.
+#'
 #' @param ... additional parameters to pass to \code{\link[freesurferformats]{write.fs.mgh}}. Only applicable for MGH and MGZ format output files, ignored for curv files.
 #'
 #' @return format, string. The format that was used to write the data. One of c("mgh", "mgz", "curv").
@@ -66,14 +68,23 @@ fwrite3 <- function(filehandle, data) {
 #' @family morphometry functions
 #'
 #' @export
-write.fs.morph <- function(filepath, data, ...) {
-    format = fs.get.morph.file.format.from.filename(filepath);
+write.fs.morph <- function(filepath, data, format='auto', ...) {
+
+    if(! format %in% c("auto", "mgh", "mgz", "curv")) {
+      stop("Format must be one of 'auto', 'mgh', 'mgz', or 'curv'.");
+    }
+
+    if(format == 'auto') {
+      format = fs.get.morph.file.format.from.filename(filepath);
+    }
     if(format == "mgh" || format == "mgz" ) {
         write.fs.mgh(filepath, data, ...);
-    } else {
+    } else if (format == "gii") {
+        stop("Writing files in GIFTI format is not supported.");
+    } else if (format == "curv") {
         write.fs.curv(filepath, data);
     }
-    return(format);
+    return(invisible(format));
 }
 
 
@@ -83,7 +94,7 @@ write.fs.morph <- function(filepath, data, ...) {
 #'
 #' @param filepath, string. A path to a file.
 #'
-#' @return format, string. The format, one of c("mgz", "mgh", "curv").
+#' @return format, string. The format, one of c("mgz", "mgh", "curv", "gii").
 #'
 #' @family morphometry functions
 #'
@@ -99,6 +110,20 @@ fs.get.morph.file.format.from.filename <- function(filepath) {
         if(tolower(ext) == "mgz") {
             return("mgz");
         }
+        if(tolower(ext) == "gii") {
+          return("gii");
+        }
+        if(tolower(ext) == ".gz") {
+            # Check whether it is '.gii.gz'
+            num_chars_to_inspect_deep = 7;
+            if(nc >= num_chars_to_inspect_deep) {
+                deep_ext = substr(filepath, nchar(filepath)-num_chars_to_inspect_deep+1, nchar(filepath));
+            }
+            if(tolower(deep_ext) == ".gii.gz") {
+                return("gii"); # The gifti reader function handles gii.gz.
+            }
+            # Otherwise we assume gzipped curv format.
+        }
     }
     return("curv");
 }
