@@ -1,12 +1,3 @@
-#' @title Determine whether a test is running on CRAN under macos
-#'
-#' @description We are currently getting failed unit tests on CRAN under macos, while the package works under MacOS on both <https://builder.r-hub.io/> and on our MacOS machines. This is because the package file cache does not work on CRAN, as the HOME is mounted read-only on the CRAN test systems. So we have to skip the tests that require optional data under MacOS on CRAN.
-#'
-#' @return logical, whether a test is running on CRAN under MacOS
-tests_running_on_cran_under_macos <- function() {
-  return(tolower(Sys.info()[["sysname"]]) == 'darwin' && !identical(Sys.getenv("NOT_CRAN"), "true"));
-}
-
 
 test_that("The dimensions of our demo 3D MGZ file are read correctly", {
   brain_image = system.file("extdata", "brain.mgz", package = "freesurferformats", mustWork = TRUE);
@@ -43,6 +34,11 @@ test_that("The demo 1D MGZ file is read correctly", {
   expect_equal(length(dim(vd)), 4);  # It has 4 dimensions
   num_vers_fsaverage = 163842
   expect_equal(dim(vd), c(num_vers_fsaverage, 1, 1, 1));
+
+  vd2 = read.fs.volume(morph_file);
+  expect_equal(class(vd2), "array");
+  expect_equal(length(dim(vd2)), 4);  # It has 4 dimensions
+  expect_equal(dim(vd2), c(num_vers_fsaverage, 1, 1, 1));
 })
 
 
@@ -51,6 +47,9 @@ test_that("The header can be read", {
   ret = read.fs.mgh(brain_image, with_header=TRUE);
   expect_true(is.fs.volume(ret));
 
+  ret2 = read.fs.volume(brain_image, with_header=TRUE);
+  expect_true(is.fs.volume(ret2));
+
   header = ret$header;
   expect_equal(class(header), "list");
   expect_equal(header$dtype, 0);  # MRI_UCHAR
@@ -72,6 +71,14 @@ test_that("The header can be read", {
   expect_equal(length(header$vox2ras_matrix), 16);
   expect_equal(header$vox2ras_matrix, matrix(c(-1,0,0,0,  0,0,-1,0,  0,1,0,0,  127.5,-98.6273,79.0953,1.000), nrow=4, byrow = FALSE), tolerance=1e-2);
 
+  header2 = ret2$header;
+  expect_equal(class(header2), "list");
+  expect_equal(header2$dtype, 0);  # MRI_UCHAR
+  expect_equal(header2$dof, 0);
+  expect_equal(header2$ras_good_flag, 1);
+  expect_equal(length(header2$internal$delta), 3);
+
+
 
   vd = ret$data;
   expect_equal(class(vd), "array");
@@ -81,6 +88,15 @@ test_that("The header can be read", {
   expect_equal(vd[80,80,80,1], 110); # value of voxel 80,80,80 must be 110 (known from ref. implementation)
   expect_equal(vd[100,100,100,1], 77);
   expect_equal(vd[100,100,80,1], 105);
+
+  vd2 = ret2$data;
+  expect_equal(class(vd2), "array");
+  expect_equal(length(dim(vd2)), 4);  # It has 4 dimensions
+  expect_equal(typeof(vd2), "integer");
+  expect_equal(dim(vd2), c(256, 256, 256, 1));
+  expect_equal(vd2[80,80,80,1], 110); # value of voxel 80,80,80 must be 110 (known from ref. implementation)
+  expect_equal(vd2[100,100,100,1], 77);
+  expect_equal(vd2[100,100,80,1], 105);
 })
 
 
